From: Ingo Molnar <mingo@elte.hu>
To: Andrew Morton <akpm@linux-foundation.org>
Cc: Jeremy Fitzhardinge <jeremy@goop.org>,
linux-kernel@vger.kernel.org,
Linus Torvalds <torvalds@linux-foundation.org>,
stable@kernel.org, Greg KH <greg@kroah.com>,
Chris Wright <chrisw@sous-sol.org>,
Jens Axboe <jens.axboe@oracle.com>
Subject: Re: [patch] fix the softlockup watchdog to actually work
Date: Wed, 25 Jul 2007 11:23:42 +0200 [thread overview]
Message-ID: <20070725092342.GA29814@elte.hu> (raw)
In-Reply-To: <20070725021730.a9a624b3.akpm@linux-foundation.org>
* Andrew Morton <akpm@linux-foundation.org> wrote:
> > * 2^30ns == 1.074s.
> > */
> > -static unsigned long get_timestamp(void)
> > +static unsigned long get_timestamp(int this_cpu)
> > {
> > - return sched_clock() >> 30; /* 2^30 ~= 10^9 */
> > + return cpu_clock(this_cpu) >> 30; /* 2^30 ~= 10^9 */
> > }
> >
> > void touch_softlockup_watchdog(void)
> > {
> > - __raw_get_cpu_var(touch_timestamp) = get_timestamp();
> > + int this_cpu = raw_smp_processor_id();
> > +
> > + per_cpu(touch_timestamp, this_cpu) = get_timestamp(this_cpu);
> > }
> > EXPORT_SYMBOL(touch_softlockup_watchdog);
> >
> > @@ -95,7 +97,7 @@ void softlockup_tick(void)
> > return;
>
> argh. afacit this was never sent, except as part of some jumbopatch
> called "sched: implement cpu_clock(cpu) high-speed time source".
>
> That patch helped.
>
> It's all a plot.
sorry, it's really my fault: i decoupled it from the jumbopatch (so that
the new API could go in first) but forgot to re-send that crutial bit.
There's also the patch below (Jens Cc:-ed) to update blktrace. I guess i
should do a softlockup.git tree to avoid such foul-ups in the future.
Ingo
----------------------->
Subject: blktrace: use cpu_clock() instead of sched_clock()
From: Ingo Molnar <mingo@elte.hu>
use cpu_clock() instead of sched_clock(). (the latter is not a proper
clock-source)
Signed-off-by: Ingo Molnar <mingo@elte.hu>
---
block/blktrace.c | 20 ++++++++++----------
1 file changed, 10 insertions(+), 10 deletions(-)
Index: linux/block/blktrace.c
===================================================================
--- linux.orig/block/blktrace.c
+++ linux/block/blktrace.c
@@ -41,7 +41,7 @@ static void trace_note(struct blk_trace
const int cpu = smp_processor_id();
t->magic = BLK_IO_TRACE_MAGIC | BLK_IO_TRACE_VERSION;
- t->time = sched_clock() - per_cpu(blk_trace_cpu_offset, cpu);
+ t->time = cpu_clock(cpu) - per_cpu(blk_trace_cpu_offset, cpu);
t->device = bt->dev;
t->action = action;
t->pid = pid;
@@ -159,7 +159,7 @@ void __blk_add_trace(struct blk_trace *b
t->magic = BLK_IO_TRACE_MAGIC | BLK_IO_TRACE_VERSION;
t->sequence = ++(*sequence);
- t->time = sched_clock() - per_cpu(blk_trace_cpu_offset, cpu);
+ t->time = cpu_clock(cpu) - per_cpu(blk_trace_cpu_offset, cpu);
t->sector = sector;
t->bytes = bytes;
t->action = what;
@@ -488,17 +488,17 @@ void blk_trace_shutdown(request_queue_t
}
/*
* Average offset over two calls to sched_clock() with a gettimeofday()
* in the middle
*/
-static void blk_check_time(unsigned long long *t)
+static void blk_check_time(unsigned long long *t, int this_cpu)
{
unsigned long long a, b;
struct timeval tv;
- a = sched_clock();
+ a = cpu_clock(this_cpu);
do_gettimeofday(&tv);
- b = sched_clock();
+ b = cpu_clock(this_cpu);
*t = tv.tv_sec * 1000000000 + tv.tv_usec * 1000;
*t -= (a + b) / 2;
@@ -510,16 +510,16 @@ static void blk_check_time(unsigned long
static void blk_trace_check_cpu_time(void *data)
{
unsigned long long *t;
- int cpu = get_cpu();
+ int this_cpu = get_cpu();
- t = &per_cpu(blk_trace_cpu_offset, cpu);
+ t = &per_cpu(blk_trace_cpu_offset, this_cpu);
/*
* Just call it twice, hopefully the second call will be cache hot
* and a little more precise
*/
- blk_check_time(t);
- blk_check_time(t);
+ blk_check_time(t, this_cpu);
+ blk_check_time(t, this_cpu);
put_cpu();
}
next prev parent reply other threads:[~2007-07-25 9:24 UTC|newest]
Thread overview: 40+ messages / expand[flat|nested] mbox.gz Atom feed top
2007-07-17 11:44 [patch] softlockup watchdog: fix Xen bogosity Ingo Molnar
2007-07-17 14:17 ` Jeremy Fitzhardinge
2007-07-17 15:49 ` [patch] fix the softlockup watchdog to actually work Ingo Molnar
2007-07-17 17:03 ` Randy Dunlap
2007-07-17 17:25 ` Ingo Molnar
2007-07-17 18:14 ` Linus Torvalds
2007-07-17 21:38 ` Randy Dunlap
2007-07-19 7:22 ` Andrew Morton
2007-07-19 7:46 ` Ingo Molnar
2007-07-19 7:51 ` Ingo Molnar
2007-07-19 14:31 ` Jeremy Fitzhardinge
2007-07-19 14:35 ` Ingo Molnar
2007-07-19 14:40 ` Jeremy Fitzhardinge
2007-07-19 14:46 ` Jeremy Fitzhardinge
2007-07-19 14:50 ` Ingo Molnar
2007-07-19 15:04 ` Jeremy Fitzhardinge
2007-07-19 15:09 ` Ingo Molnar
2007-07-19 15:21 ` Jeremy Fitzhardinge
2007-07-19 15:42 ` [patch] sched: implement cpu_clock(cpu) high-speed time source Ingo Molnar
2007-07-19 15:44 ` [patch] sched: implement cpu_clock(cpu) high-speed time source, take #2 Ingo Molnar
2007-07-19 16:11 ` Jeremy Fitzhardinge
2007-07-19 16:16 ` Ingo Molnar
2007-07-19 16:18 ` Jeremy Fitzhardinge
2007-07-19 16:21 ` Ingo Molnar
2007-07-19 16:29 ` Jeremy Fitzhardinge
2007-07-19 17:24 ` Jens Axboe
2007-07-19 18:10 ` Jeremy Fitzhardinge
2007-07-19 18:20 ` Jens Axboe
2007-07-25 8:49 ` [patch] fix the softlockup watchdog to actually work Andrew Morton
2007-07-25 8:52 ` Ingo Molnar
2007-07-25 8:55 ` Ingo Molnar
2007-07-25 9:00 ` Andrew Morton
2007-07-25 9:04 ` Ingo Molnar
2007-07-25 9:17 ` Andrew Morton
2007-07-25 9:23 ` Ingo Molnar [this message]
2007-07-25 9:59 ` Jens Axboe
2007-07-25 11:04 ` Ingo Molnar
2007-07-25 11:06 ` Jens Axboe
2007-07-25 16:34 ` Andi Kleen
2007-10-03 23:49 ` Yinghai Lu
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20070725092342.GA29814@elte.hu \
--to=mingo@elte.hu \
--cc=akpm@linux-foundation.org \
--cc=chrisw@sous-sol.org \
--cc=greg@kroah.com \
--cc=jens.axboe@oracle.com \
--cc=jeremy@goop.org \
--cc=linux-kernel@vger.kernel.org \
--cc=stable@kernel.org \
--cc=torvalds@linux-foundation.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -488,17 +488,17 @@ void blk_trace_shutdown(request_queue_t
}
/*
- * Average offset over two calls to sched_clock() with a gettimeofday()
+ * Average offset over two calls to cpu_clock() with a gettimeofday()
* in the middle
*/
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
Powered by JetHome